#!/usr/bin/perl

package Cognition::Export::Feed;
use Cognition::Export;
use Cognition::Export::BlogEntry;
use HTML::Entities qw(encode_entities_numeric);
use DateTime;

sub is
{
	my $subject = shift;
	return unless (ref $subject eq 'Cognition::RDFModel::Subject');	
	return
		unless (defined $subject->property('http://www.w3.org/1999/02/22-rdf-syntax-ns#type'));
	foreach my $Type ( @{$subject
		->property('http://www.w3.org/1999/02/22-rdf-syntax-ns#type')
		->valObjects} )
	{
		my $type = $Type->toString;
		return 1
			if (($type eq 'http://purl.org/rss/1.0/channel')
			||  ($type eq 'urn:ietf:rfc:4287#feed')
			||  ($type eq 'http://purl.invalid/stuff/atom/owl#Feed')
			||  ($type eq 'http://bblfish.net/work/atom-owl/2006-06-06/#Feed')
			);			
	}
	
	return 1 if (firstOf($subject,
		'http://purl.org/rss/1.0/item',
		'http://purl.org/rss/1.0/items',
		'urn:ietf:rfc:4287#entry',
		'http://purl.invalid/stuff/atom/owl#entry',
		'http://bblfish.net/work/atom-owl/2006-06-06/#entry'
		));
	
	return;
}

sub outputs
{
	return [
		{
			code  => 'Atom',
			name  => 'Atom Feed',
			type  => 'application/atom+xml',
			multi => 1
		}
	];
}

sub multiAtom
{
	my $triples = shift;
	my $rv = '';
	foreach my $s (keys %$triples)
	{
		$rv .= outAtom($triples, $s) . "\n\n"
			if (defined $triples->{$s} && is($triples->{$s}));
	}
	return $rv;
}

sub outAtom
{
	my $triples = shift;
	my $subject = shift;
	my $S = $triples->{$subject};
	my $rv = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n";
	$rv .= "<feed xmlns=\"http://www.w3.org/2005/Atom\"\n";
	$rv .= "      xmlns:thr=\"http://purl.org/syndication/thread/1.0\">\n";
	$rv .= "\t<generator>cognition</generator>\n";
	
	my $link = firstOf($S,
		'http://www.iana.org/assignments/relation/self',
		'urn:ietf:rfc:4287#link',
		'http://purl.org/rss/1.0/link',
		'http://www.w3.org/2006/link#uri');
	$rv .= "\t<id>".encode_entities_numeric($link)."</id>\n";
	$rv .= "\t<link rel=\"alternate\" href=\"".encode_entities_numeric($link)."\" />\n";

	my $title = firstOfLiteral($S,
		['http://purl.invalid/stuff/atom/owl#title',
		'http://bblfish.net/work/atom-owl/2006-06-06/#title',
		'urn:ietf:rfc:4287#title',
		'http://purl.org/rss/1.0/title',
		'http://purl.org/dc/terms/title',
		'http://purl.org/dc/elements/1.1/title'],
		['urn:ietf:rfc:4287#value',
		'urn:ietf:rfc:2445#value',
		'http://www.w3.org/1999/02/22-rdf-syntax-ns#value']);
	$title = 'Untitled' if (!$title);
	$rv .= "\t<title>".encode_entities_numeric($title)."</title>\n";
	
	my $subtitle = firstOfLiteral($S,
		['http://purl.invalid/stuff/atom/owl#subtitle',
		'http://bblfish.net/work/atom-owl/2006-06-06/#subtitle',
		'urn:ietf:rfc:4287#subtitle',
		'http://purl.org/dc/terms/abstract'],
		['http://purl.invalid/stuff/atom/owl#xhtml',
		'http://bblfish.net/work/atom-owl/2006-06-06/#xhtml',
		'http://purl.invalid/stuff/atom/owl#html',
		'http://bblfish.net/work/atom-owl/2006-06-06/#html',
		'http://purl.invalid/stuff/atom/owl#text',
		'http://bblfish.net/work/atom-owl/2006-06-06/#text',
		'http://www.w3.org/1999/02/22-rdf-syntax-ns#value']);
	$rv .= "\t<subtitle>".encode_entities_numeric($subtitle)."</subtitle>\n"
		if ($subtitle);

	my @authors = allOfObj($S,
		'http://purl.invalid/stuff/atom/owl#author',
		'http://bblfish.net/work/atom-owl/2006-06-06/#author',
		'urn:ietf:rfc:4287#author',
		'http://purl.org/dc/terms/creator',
		'http://purl.org/dc/elements/1.1/creator',
		'http://xmlns.com/foaf/0.1/maker');
	foreach my $author (@authors)
	{
		my $authorSubject = findSubject($triples, $author);
		if ($authorSubject && Cognition::Export::Contact::is($authorSubject))
		{
			$rv .= Cognition::Export::Contact::_AtomPerson(
				$triples,
				$authorSubject->uri,
				"\t<author>");
		}
		elsif ($author->type)
		{
			$rv .= "\t<author><url>".$author->toString."</url></author>\n";
		}
		else
		{
			$rv .= "\t<author><name>".$author->toString."</name></author>\n";
		}
	}

	my @contributors = allOfObj($S,
		'http://purl.invalid/stuff/atom/owl#contributor',
		'http://bblfish.net/work/atom-owl/2006-06-06/#contributor',
		'urn:ietf:rfc:4287#contributor',
		'http://purl.org/dc/terms/contributor',
		'http://purl.org/dc/elements/1.1/contributor');
	foreach my $contributor (@contributors)
	{
		my $contributorSubject = findSubject($triples, $contributor);
		if ($contributorSubject && Cognition::Export::Contact::is($contributorSubject))
		{
			$rv .= Cognition::Export::Contact::_AtomPerson(
				$triples,
				$contributorSubject->uri,
				"\t<contributor>");
		}
		elsif ($contributor->type)
		{
			$rv .= "\t<contributor><url>".$contributor->toString."</url></contributor>\n";
		}
		else
		{
			$rv .= "\t<contributor><name>".$contributor->toString."</name></contributor>\n";
		}
	}

	$rv .= "\t<updated>UPDATED_PLACEHOLDER_99754106633f94d350db34d548d6091a</updated>\n";

	my @cats = allOfObj($S,
		'http://purl.invalid/stuff/atom/owl#category',
		'http://bblfish.net/work/atom-owl/2006-06-06/#category',
		'urn:ietf:rfc:4287#category',
		'http://www.holygoat.co.uk/owl/redwood/0.1/tags/taggedWithTag');
	foreach my $cat (@cats)
	{
		my ($tag, $scheme, $label);
		my $catSubject = findSubject($triples, $cat->toString);
		
		if ($catSubject)
		{
			$tag    = firstOf($catSubject,
				'http://purl.invalid/stuff/atom/owl#term',
				'http://bblfish.net/work/atom-owl/2006-06-06/#term',
				'http://www.holygoat.co.uk/owl/redwood/0.1/tags/name',
				'http://www.w3.org/1999/02/22-rdf-syntax-ns#value',
				'http://purl.org/dc/terms/title',
				'http://purl.org/dc/elements/1.1/title');
			$scheme = firstOf($catSubject,
				'http://purl.invalid/stuff/atom/owl#scheme',
				'http://bblfish.net/work/atom-owl/2006-06-06/#scheme');
			$label  = firstOf($catSubject,
				'http://purl.invalid/stuff/atom/owl#label',
				'http://bblfish.net/work/atom-owl/2006-06-06/#label',
				'http://www.w3.org/2000/01/rdf-schema#label',
				'http://www.holygoat.co.uk/owl/redwood/0.1/tags/name',
				'http://www.w3.org/1999/02/22-rdf-syntax-ns#value',
				'http://purl.org/dc/terms/title',
				'http://purl.org/dc/elements/1.1/title');
		}
		$tag   = $cat->toString if (!$tag);
		$label = $cat->toString if (!$label);
		
		$rv .= "\t<category term=\"".encode_entities_numeric($tag)."\" ";
		$rv .= "scheme=\"".encode_entities_numeric($scheme)."\" "
			if (length $scheme);
		$rv .= "label=\"".encode_entities_numeric($label)."\" "
			if (length $label);
		$rv .= "/>\n";
	}

	my $rights = firstOfLiteral($S,
		['http://purl.invalid/stuff/atom/owl#rights',
		'http://bblfish.net/work/atom-owl/2006-06-06/#rights',
		'urn:ietf:rfc:4287#rights',
		'http://purl.org/dc/terms/rights',
		'http://purl.org/dc/elements/1.1/rights'],
		['http://purl.invalid/stuff/atom/owl#xhtml',
		'http://bblfish.net/work/atom-owl/2006-06-06/#xhtml',
		'http://purl.invalid/stuff/atom/owl#html',
		'http://bblfish.net/work/atom-owl/2006-06-06/#html',
		'http://purl.invalid/stuff/atom/owl#text',
		'http://bblfish.net/work/atom-owl/2006-06-06/#text',
		'http://www.w3.org/1999/02/22-rdf-syntax-ns#value']);
	$rv .= "\t\t<rights>".encode_entities_numeric($rights)."</rights>\n"
		if ($rights);
	
	my $updatedLatest;
	foreach my $e (allOf($S,
		'http://purl.invalid/stuff/atom/owl#entry',
		'http://bblfish.net/work/atom-owl/2006-06-06/#entry',
		'http://purl.org/rss/1.0/item',
		'urn:ietf:rfc:4287#entry'))
	{
		$rv .= Cognition::Export::BlogEntry::_outAtom($triples, $e, $updatedLatest);
	}

	my $updated = firstOf($S,
		'http://purl.invalid/stuff/atom/owl#updated',
		'http://bblfish.net/work/atom-owl/2006-06-06/#updated',
		'urn:ietf:rfc:4287#updated',
		'http://purl.org/dc/terms/modified',
		'http://purl.org/dc/terms/date',
		'http://purl.org/dc/elements/1.1/date');
	if (!$updated)
	{
		$updated = $updatedLatest;
		$updated = DateTime->now()
			if (!$updated);
	}
	if ($updated)
	{
		$updated =~ s/(\d\d)(\d\d)$/$1:$2/;
		$rv =~ s/UPDATED_PLACEHOLDER_99754106633f94d350db34d548d6091a/$updated/;
	}
	
	$rv .= "</feed>\n";
	return $rv;
}

1;
