#!/usr/bin/perl

######################################################################
package Cognition::HTMLParser::Metadata;
######################################################################

use Cognition::HTMLParser::HTTP;
use Cognition::MagicString;
use XML::LibXML qw(:all);

sub is_enabled
{
	return 1;
}

sub consume
{
	my $page = shift;

	# Parse <title>.
	my $S = Cognition::RDFModel::Subject->new($page->uri);
	foreach my $t ($page->{DOM}->getElementsByTagName('title'))
	{
		my $val = Cognition::RDFModel::Value->new(Cognition::HTMLParser::Utils::STRINGIFY($t));
		$val->setDom($t);
		$S->addVal('http://purl.org/dc/terms/title', $val);
	}
	my $val = Cognition::RDFModel::Value->new('http://purl.org/dc/dcmitype/Text', 1);
	$val->setVocab('http://purl.org/dc/terms/DCMIType');
	$S->addVal('http://www.w3.org/1999/02/22-rdf-syntax-ns#type', $val);
	$page->mergeSubject($S);
	
	meta_tags($page);
	link_tags($page);
}

sub link_tags
{
	my $this = shift;
	my $myurl = $this->uri('');
	
	# Get list of links.
	my @links   = $this->{DOM}->getElementsByTagName('link');
	my @anchors = $this->{DOM}->getElementsByTagName('a');
	push @links, @anchors;
	
	# For each link
	foreach my $l (@links)
	{
		# Subject: Linked page
		my $href = $this->uri($l->getAttribute('href'));
		next unless (length $href);
		my $SubjectLinkedPage;
		
		# Handle @title.
		if (length $l->getAttribute('title'))
		{
			$SubjectLinkedPage = Cognition::RDFModel::Subject->new($href)
				unless ($SubjectLinkedPage);

			my $val = Cognition::RDFModel::Value->new($l->getAttribute('title'));
			$val->setDom($l);
			$SubjectLinkedPage->addVal('http://purl.org/dc/terms/title', $val);
		}

		# Handle @media.
		if (length $l->getAttribute('media'))
		{
			$SubjectLinkedPage = Cognition::RDFModel::Subject->new($href)
				unless ($SubjectLinkedPage);

			my $val = Cognition::RDFModel::Value->new($l->getAttribute('media'));
			$val->setDom($l);
#			$val->setVocab('http://www.w3.org/TR/CSS21/media.html#media-types');
			$SubjectLinkedPage->addVal('http://purl.org/dc/terms/format', $val);
		}

		# Handle @title.
		if (length $l->getAttribute('type'))
		{
			$SubjectLinkedPage = Cognition::RDFModel::Subject->new($href)
				unless ($SubjectLinkedPage);

			my $val = Cognition::RDFModel::Value->new($l->getAttribute('type'));
			$val->setDom($l);
			$val->setVocab('http://purl.org/dc/terms/IMT');
			$SubjectLinkedPage->addVal('http://purl.org/dc/terms/format', $val);
		}

		# Handle @lang/@hreflang.
		if (length $l->getAttribute('lang')
		||  length $l->getAttribute('xml:lang')
		||  length $l->getAttribute('hreflang'))
		{
			$SubjectLinkedPage = Cognition::RDFModel::Subject->new($href)
				unless ($SubjectLinkedPage);

			my $val = Cognition::RDFModel::Value->new(
				$l->getAttribute('hreflang') ||
				$l->getAttribute('xml:lang') ||
				$l->getAttribute('lang')
			);
			$val->setDom($l);
			$val->setVocab('http://purl.org/dc/terms/RFC4646');
			$SubjectLinkedPage->addVal('http://purl.org/dc/terms/language', $val);
		}

		$this->mergeSubject($SubjectLinkedPage);
	}
}

sub meta_tags
{
	my $this = shift;
	my $myurl = $this->uri;

	# Subject: This page
	my $SubjectThisPage = Cognition::RDFModel::Subject->new($myurl);		

	my $nsrdf   = 'http://www.w3.org/1999/02/22-rdf-syntax-ns#';
	my $nshttp  = 'http://www.w3.org/2006/http#';
	my $nshttpH = 'http://www.w3.org/2008/http-headers#';
	my $nshttpM = 'http://www.w3.org/2008/http-methods#';
	my $nshttpS = 'http://www.w3.org/2008/http-statusCodes#';
	my $rdftype = $nsrdf.'type';

	$this->mergeNS('ianalink',
		'http://www.iana.org/assignments/relation/',
		'IANA Registered Link Types',
		-1,
		0);
	$this->mergeNS('http',
		$nshttp,
		'HTTP Vocabulary in RDF',
		-1,
		0);
	$this->mergeNS('http-ext',
		'http://buzzword.org.uk/rdf/http-ext#',
		'HTTP Vocabulary in RDF Extentions',
		-1,
		0);
	$this->mergeNS('httphdr',
		$nshttpH,
		'HTTP Vocabulary in RDF (Headers)',
		-1,
		0);
	$this->mergeNS('httpmeth',
		$nshttpM,
		'HTTP Vocabulary in RDF (Methods)',
		-1,
		0);
	$this->mergeNS('httpstat',
		$nshttpS,
		'HTTP Vocabulary in RDF (Status Codes)',
		-1,
		0);

	my $HTTPEq = Cognition::RDFModel::Subject->new(
		Cognition::HTMLParser::Utils::BNodeURI(undef, 'HttpEquivHeaders'));
	$HTTPEq->addVal($rdftype, Cognition::RDFModel::Value->new($nsrdf.'List', 1));

	my $i = 1;

	# Parse <meta>
	foreach my $m ($this->{DOM}->getElementsByTagName('meta'))
	{
		# Value
		my $content = length($m->getAttribute('content')) ?
		                   $m->getAttribute('content') :
		                   Cognition::HTMLParser::Utils::STRINGIFY($m);

		# @http-equiv
		my $name = $m->getAttribute('http-equiv');
		if (length $name)
		{
			my $huri = Cognition::HTMLParser::HTTP::consume_header($this, $SubjectThisPage, $name, $content);
			$HTTPEq->addVal('http://buzzword.org.uk/rdf/list-items/_'.$i,
				Cognition::RDFModel::Value->new($huri, 1));
			$i++;
		}
		
		# @name
		else
		{
			$name = $m->getAttribute('name');
			next unless (length $name);

			$name =~ s/(^\s+|\s+$)//g;
			$name =~ s/\s+/ /g;
			my @name = split / /, $name;
			
			foreach my $n (@name)
			{
				my $nsname = reservedWordOrCurie($this, $n, $m);
				my $val    = Cognition::RDFModel::Value->new($content);
				$val->setDom($m);
				if ($m->getAttribute('scheme'))
				{
					my $nsscheme = reservedWordOrCurie($this, $m->getAttribute('scheme'), $m);
					$val->setVocab($nsscheme);
				}
				$SubjectThisPage->addVal($nsname, $val);
			}
		}
	}
	
	if ($i > 1)
	{
		$this->mergeSubject($HTTPEq);
		$SubjectThisPage->addVal('http://buzzword.org.uk/rdf/http-ext#http-equiv-headers',
			Cognition::RDFModel::Value->new($HTTPEq->uri, 1));
	}

	$this->mergeSubject($SubjectThisPage);
}


sub reservedWordOrCurie
{
	my $page  = shift;
	my $str   = shift;
	my $dom   = shift;
	my $xpath = $dom->getAttribute('_xpath') if ($dom);
	my $safe  = 0;
	
	if ($str =~ /^(application-name|description|generator|cache|keywords|robots)$/i)
	{
		return 'http://buzzword.org.uk/rdf/h5#'.lc($1);
	}
	
	# Cope with safe CURIEs.
	if ($str =~ /^\[(.*)\]$/)
	{
		$str  = $1;
		$safe = 1;
	}
	
	# If the string matches CURIE syntax, then resolve the CURIE.
	# This function allows a slightly looser CURIE syntax. Allows dots.
	if ($str =~ /^([a-z_][^\s\:\.]*)[\:\.](.*)$/i)
	{
		my $pfx = $1;
		my $sfx = $2;
		my $ns  = Cognition::Namespace::bestMatch($pfx, $xpath, @{$page->{Namespaces}});
		
		return $page->uri( $ns->{uri} . $sfx )
			if ($ns);
			
		return 'http://invalid.invalid/ns#'.$sfx
			if ($safe);
	}
	
	# If it wasn't a safe CURIE, then fall back to a URI.
	if (!$safe)
	{
		return $page->uri($str, {'require-absolute'=>1,'element'=>$dom});
	}
	
	return undef;
}


1;
